package l17_swing;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class BouncyBall extends JPanel {

	public BouncyBall() {

		final BallPitPane ballPitPane = new BallPitPane();
		final JPanel outerPanel = new JPanel();
		outerPanel.setLayout(new BorderLayout());
		outerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		outerPanel.add(ballPitPane, BorderLayout.CENTER);

		setLayout(new BorderLayout());
		add(outerPanel, BorderLayout.CENTER);

		JPanel controls = new JPanel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(10, 10, 10, 10);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;

		gbc.gridy = 0;
		JSlider speed = new SliderWithLabel(0, 100, 1, controls, gbc, "Speed").getSlider();
		speed.addChangeListener(e -> ballPitPane.setSpeed(speed.getValue()));
		gbc.gridy = 1;
		JSlider quantity = new SliderWithLabel(0, 100, 1, controls, gbc, "Quantity").getSlider();
		quantity.addChangeListener(e -> ballPitPane.setQuantity(quantity.getValue()));
		add(controls, BorderLayout.PAGE_END);
	}

	public static class SliderWithLabel {

		private final JSlider slider;

		public SliderWithLabel(int min, int max, int value, JPanel panel, GridBagConstraints gbc, String text) {
			slider = new JSlider(min, max, value);
			slider.setMajorTickSpacing(25);
			slider.setMinorTickSpacing(5);
			slider.setPaintLabels(true);
			slider.setPaintTicks(true);
			gbc.gridx = 0;
			gbc.weightx = 0;
			panel.add(new JLabel(STR."\{text}: "), gbc);
			gbc.gridx = 1;
			gbc.weightx = 1;
			panel.add(slider, gbc);
		}

		public JSlider getSlider() {
			return slider;
		}
	}

	public static class BallPitPane extends JPanel {

		private final List<Ball> balls = new ArrayList<>();
		private int speed = 2;
		private static final Random random = new Random();
		private static final int timerDelay = 40;

		public BallPitPane() {
			setLayout(new BorderLayout());
			setBackground(Color.white);
			setBorder(BorderFactory.createLineBorder(Color.BLACK, 2));
			new Timer(timerDelay, e -> repaint()).start();
		}

		@Override
		public Dimension getPreferredSize() {
			return new Dimension(600, 400);
		}

		public void setSpeed(int speed) {
			this.speed = speed;
		}

		public void setQuantity(int quantity) {
			if (balls.size() > quantity) {
				balls.subList(0, balls.size() - quantity).clear();
			}
			while (balls.size() < quantity) {
				int diameter = random.nextInt(16, 65);
				balls.add(new Ball(
					randomColor(),
					// in case the window size < diameter
					random.nextInt(Math.abs(getWidth() - diameter)),
					random.nextInt(Math.abs(getHeight() - diameter)),
					diameter
				));
			}
		}

		private Color randomColor() {
			return new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256));
		}

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			if (balls.isEmpty()) setQuantity(1);
			for (Ball ball : balls) {
				ball.update(getWidth(), speed);
				ball.paint(g);
			}
		}

		public static class Ball {

			private final Color color;
			private int x;
			private final int y;
			private final int diameter;
			private int direction;

			public Ball(Color color, int x, int y, int diameter) {
				this.color = color;
				this.x = x;
				this.y = y;
				this.diameter = diameter;
				direction = random.nextInt(2) * 2 - 1;
			}

			public void update(int width, int speed) {
				x += speed * direction;
				if (x + diameter >= width) {
					x = width - diameter;
					direction = -direction;
				} else if (x < 0) {
					x = 0;
					direction = -direction;
				}
			}

			public void paint(Graphics g) {
				g.setColor(color);
				g.fillOval(x, y, diameter, diameter);
				g.setColor(Color.BLACK);
				g.drawOval(x, y, diameter, diameter);
			}
		}
	}

	public static void main(String[] args) {
		SwingConsole.run(new BouncyBall());
	}
}

