package l17_swing;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.function.DoubleBinaryOperator;

public class Calculator extends JPanel {
	private final JTextField resultsTxt;
    private Double leftOperand;
	private Double rightOperand;
	private Operation calcOperation;

    public Calculator() {

		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(10,10,10,10));

        // set results panel
		JPanel resultsPanel = new JPanel();
		resultsPanel.setLayout(new GridLayout(1,1));
		resultsPanel.setBorder(new EmptyBorder(0, 0, 10, 0));
		resultsTxt = new JTextField();
		resultsTxt.setFont(new Font("Courier", Font.PLAIN, 16));
        resultsPanel.add(resultsTxt);

		// set button panel
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(0, 4, 10, 10));
        JButton[] numButtons = new JButton[10];
        for (int i = 0; i < 10; i++) {
			numButtons[i] = new JButton(String.valueOf(i));
			numButtons[i].addActionListener(new NumberBtnClicked(numButtons[i].getText()));
		}
        JButton clearBtn = new JButton("AC");
        JButton signBtn = new JButton("+/-");
        JButton moduloBtn = new JButton("%");
        JButton divideBtn = new JButton("/");
        JButton multiplyBtn = new JButton("X");
        JButton minusBtn = new JButton("-");
        JButton addBtn = new JButton("+");
        JButton equalBtn = new JButton("=");
        JButton decimalBtn = new JButton(".");

		moduloBtn.addActionListener(new OperationBtnClicked(Operation.MODULO));
		multiplyBtn.addActionListener(new OperationBtnClicked(Operation.MULTIPLICATION));
		divideBtn.addActionListener(new OperationBtnClicked(Operation.DIVISION));
		minusBtn.addActionListener(new OperationBtnClicked(Operation.SUBTRACTION));
		addBtn.addActionListener(new OperationBtnClicked(Operation.ADDITION));
		equalBtn.addActionListener(new EqualBtnClicked());
		clearBtn.addActionListener(new ClearBtnClicked());
		signBtn.addActionListener(new SignBtnClicked());
		decimalBtn.addActionListener(new PointBtnClicked());

		buttonPanel.add(clearBtn);
		buttonPanel.add(signBtn);
		buttonPanel.add(moduloBtn);
		buttonPanel.add(divideBtn);
		buttonPanel.add(numButtons[7]);
		buttonPanel.add(numButtons[8]);
		buttonPanel.add(numButtons[9]);
		buttonPanel.add(multiplyBtn);
		buttonPanel.add(numButtons[4]);
		buttonPanel.add(numButtons[5]);
		buttonPanel.add(numButtons[6]);
		buttonPanel.add(minusBtn);
		buttonPanel.add(numButtons[1]);
		buttonPanel.add(numButtons[2]);
		buttonPanel.add(numButtons[3]);
		buttonPanel.add(addBtn);
		buttonPanel.add(numButtons[0]);
		buttonPanel.add(equalBtn);
		buttonPanel.add(new JLabel()); // fill empty grid
		buttonPanel.add(decimalBtn);

		// Create/configure split pane
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setContinuousLayout(true);
		splitPane.setOneTouchExpandable(true);

		// Create top component
		splitPane.setTopComponent(resultsPanel);
		// Create bottom component
		splitPane.setBottomComponent(buttonPanel);

		add(splitPane, BorderLayout.CENTER);

//		add(resultsPanel, BorderLayout.PAGE_START);
//		add(new JSeparator(), BorderLayout.CENTER);
//		add(buttonPanel, BorderLayout.PAGE_END);
	}

	private class NumberBtnClicked implements ActionListener {

		private final String value;

		public NumberBtnClicked(String value) {
			this.value = value;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
            resultsTxt.setText(resultsTxt.getText() + value);
		}
	}

	private class OperationBtnClicked implements ActionListener {

		private final Operation operation;

		public OperationBtnClicked(Operation operation) {
			this.operation = operation;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			calcOperation = operation;
			leftOperand = Double.valueOf(resultsTxt.getText());
			resultsTxt.setText("");
		}
	}

	private class ClearBtnClicked implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			resultsTxt.setText("");
			leftOperand = null;
			rightOperand = null;
		}
	}

	private class PointBtnClicked implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			resultsTxt.setText(resultsTxt.getText() + ".");
		}
	}

	private class EqualBtnClicked implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
            rightOperand = Double.valueOf(resultsTxt.getText());
			Double output = calcOperation.getOperator().applyAsDouble(leftOperand, rightOperand);
			resultsTxt.setText(output % 1 == 0 ? String.valueOf(output.intValue()) : String.valueOf(output));
			leftOperand = null;
			rightOperand = null;
		}
	}

	private class SignBtnClicked implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			if (resultsTxt.getText().startsWith("-")) {
				resultsTxt.setText(resultsTxt.getText().substring(1));
			} else {
				resultsTxt.setText("-" + resultsTxt.getText());
			}
		}
	}

	private enum Operation {
		ADDITION(Double::sum),
		SUBTRACTION((x, y) -> x - y),
		DIVISION((x, y) -> x / y),
		MULTIPLICATION((x, y) -> x * y),
		MODULO((x, y) -> x % y);


		private final DoubleBinaryOperator operator;

		Operation(DoubleBinaryOperator operator) {
			this.operator = operator;
		}

		public DoubleBinaryOperator getOperator() {
			return operator;
		}
	}

	public static void main(String[] args) {
		SwingConsole.run(new Calculator());
	}
}

