package l17_swing;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;

import javax.swing.*;
import java.awt.*;
import java.util.concurrent.*;

public class ColorBoxes extends JPanel {

	static class ColorBox extends JPanel implements Runnable {
		private final int pause;

		public ColorBox(int pause) {
			this.pause = pause;
		}

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			g.setColor(new Color(RandomGenerators.getIntThread(0xFFFFFF)));
			g.fillRect(0, 0, getWidth(), getHeight());
		}

		@Override
		public void run() {
			while(!Thread.interrupted()) {
				repaint();
				ThreadHelperMethods.safeSleep(pause);
			}
		}
	}

	public ColorBoxes() {} // necessary due to IntelliJ bug

	public ColorBoxes(int grid, int pause) {
		final int gap = 5;
		setLayout(new GridLayout(grid, grid, gap, gap));
		setBorder(BorderFactory.createEmptyBorder(gap, gap, gap, gap));
		ExecutorService executor = Executors.newFixedThreadPool(grid * grid);
		for (int i = 0; i < grid * grid; i++) {
			ColorBox colorBox = new ColorBox(pause);
			add(colorBox);
			executor.submit(colorBox);
		}
		executor.shutdown();
	}

	public static void main(String[] args) {
		int grid = (args.length > 0) ? Integer.parseInt(args[0]) : 6;
		int pause = (args.length > 1) ? Integer.parseInt(args[1]) : 2000;
		SwingConsole.run(new ColorBoxes(grid, pause), 800, 800);
	}
}

