package l17_swing;

import javax.swing.*;
import java.awt.*;

public class GradientBorder extends JPanel {

	private static final int GRADIENT_LENGTH = 30;
	private static final int WINDOW_WIDTH = 300;
	private static final int WINDOW_HEIGHT = 300;

	public static void main(String[] args) {
		SwingConsole.run(new GradientBorder(), 400, 400);
	}

	@Override
	public void paintComponent(Graphics graphics) {

		super.paintComponent(graphics);
		paintGradientBorder(graphics, Color.red, Color.white,
			WINDOW_WIDTH, WINDOW_HEIGHT, GRADIENT_LENGTH);
	}

	@SuppressWarnings("SameParameterValue")
	public static void paintGradientBorder(Graphics g, Color innerBorderColor,
			   Color outerBorderColor, int windowWidth, int windowHeight, int gradientSize) {

		Graphics2D g2 = (Graphics2D) g.create();

		GradientPaint gradientColorWest = new GradientPaint(0, 0, outerBorderColor,
			gradientSize, 0, innerBorderColor);
		GradientPaint gradientColorEast = new GradientPaint(windowWidth - gradientSize,
			0, innerBorderColor, windowWidth, 0, outerBorderColor);
		GradientPaint gradientColorNorth= new GradientPaint(0, 0, outerBorderColor, 0,
			gradientSize, innerBorderColor);
		GradientPaint gradientColorSouth = new GradientPaint(0, windowHeight - gradientSize,
			innerBorderColor,0, windowHeight, outerBorderColor);

		//south bar
		g2.setPaint(gradientColorSouth);
		g2.fillRect(0, windowHeight - gradientSize, windowWidth, gradientSize);
		//north bar
		g2.setPaint(gradientColorNorth);
		g2.fillRect(0, 0, windowWidth, gradientSize);
		//west bar
		g2.setPaint(gradientColorWest);
		g2.fillRect(0, gradientSize, gradientSize, windowHeight - gradientSize * 2);
		//east bar
		g2.setPaint(gradientColorEast);
		g2.fillRect(windowWidth - gradientSize, gradientSize, gradientSize,
			windowHeight - gradientSize * 2);

		//NORTH WEST CORNER
		//left triangle
		Polygon p = new Polygon();
		p.addPoint(0, 0);
		p.addPoint(gradientSize, gradientSize);
		p.addPoint(0, gradientSize);
		g2.setPaint(gradientColorWest);
		g2.fillPolygon(p);

		//NORTH EAST CORNER
		//right triangle
		p.reset();
		p.addPoint(windowWidth, 0);
		p.addPoint(windowWidth - gradientSize, gradientSize);
		p.addPoint(windowWidth, gradientSize);
		g2.setPaint(gradientColorEast);
		g2.fillPolygon(p);

		//SOUTH WEST CORNER
		//left triangle
		p.reset();
		p.addPoint(0, windowHeight);
		p.addPoint(0,windowHeight - gradientSize);
		p.addPoint(gradientSize, windowHeight - gradientSize);
		g2.setPaint(gradientColorWest);
		g2.fillPolygon(p);

		//SOUTH EAST CORNER
		//right triangle
		p.reset();
		p.addPoint(windowWidth, windowHeight);
		p.addPoint(windowWidth, windowHeight - gradientSize);
		p.addPoint(windowWidth - gradientSize, windowHeight - gradientSize);
		g2.setPaint(gradientColorEast);
		g2.fillPolygon(p);

		g2.dispose();
	}
}
