package l17_swing;

import javax.swing.*;
import java.awt.*;

public class GridLayoutDemo extends JPanel {
	private static final String[] gapList = {"0", "10", "15", "20"};
	private static final int maxGap = 20;
	private static final int maxButtons = 7;
	private static final int noRows = (maxButtons + 1) / 2;

//	A component that combines a button or editable field and a drop-down list.
//	The user can select a value from the drop-down list, which appears at the
//	user's request. If you make the combo box editable, then the combo box
//	includes an editable field into which the user can type a value. 

	public GridLayoutDemo() {

		final GridLayout gridLayout = new GridLayout(0,2);
		final JPanel buttonPanel = new JPanel(gridLayout);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		//Add buttons to experiment with Grid Layout
		for (int i = 0; i < maxButtons - 1; i++) {
			buttonPanel.add(new JButton(STR."Button \{i}"));
		}
		JButton longButton = new JButton(STR."Long-Named Button \{maxButtons - 1}");
		buttonPanel.add(longButton);

		//Set up components preferred size (so it looks ok after applying max gaps)
		Dimension buttonSize = longButton.getPreferredSize();
		buttonPanel.setPreferredSize(new Dimension(
			(int)(buttonSize.getWidth() * 2) + maxGap,
			(int)(buttonSize.getHeight() * noRows) + maxGap * (noRows - 1))
		);

		//Add controls to set up horizontal and vertical gaps
		JPanel controlPanel = new JPanel(new GridLayout(2,3));
		controlPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		controlPanel.add(new Label("Horizontal gap:"));
		controlPanel.add(new Label("Vertical gap:"));
		controlPanel.add(new Label(""));
		JComboBox<String> horGapComboBox = new JComboBox<>(gapList);
		JComboBox<String> verGapComboBox = new JComboBox<>(gapList);
		controlPanel.add(horGapComboBox);
		controlPanel.add(verGapComboBox);
		JButton applyButton = new JButton("Apply gaps");
		controlPanel.add(applyButton);

		// Add ActionListener to the button
		applyButton.addActionListener(e -> {
			// Set up the horizontal gap value
			gridLayout.setHgap(Integer.parseInt(STR."\{horGapComboBox.getSelectedItem()}"));
			// Set up the vertical gap value
			gridLayout.setVgap(Integer.parseInt(STR."\{verGapComboBox.getSelectedItem()}"));
			// Apply new layout
			gridLayout.layoutContainer(buttonPanel);
		});

		setLayout(new BorderLayout());
		add(buttonPanel, BorderLayout.NORTH);
		add(new JSeparator(SwingConstants.HORIZONTAL), BorderLayout.CENTER);
		add(controlPanel, BorderLayout.SOUTH);
	}

	public static void main(String[] args) {
		SwingConsole.run(new GridLayoutDemo());
	}
}

