package l17_swing;

import javax.swing.*;
import java.awt.*;

public class SineWave extends JPanel {

	private static final Color backgroundColor = new Color(247, 247, 253);

	static class SinePanel extends JPanel {
		private static final int SCALE_FACTOR = 512; // number of points per interval of length pi
		private int noOfSegments;

		public SinePanel(int  initCycles) {
			setBackground(backgroundColor);
			setCycles(initCycles);
		}

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			int margin = 32;
			int margin2 = margin / 2;
			double width = getWidth() - margin;
			double height = getHeight() - margin;
			double hStep = width / noOfSegments;
			int[] sineValues = new int[noOfSegments + 1];
			for(int i = 0; i <= noOfSegments; i++) {
				double radians = (Math.PI / SCALE_FACTOR) * i;
				sineValues[i] = (int)((1. - Math.sin(radians)) * height / 2);
			}
			Graphics2D g2 = (Graphics2D) g;
			g2.setStroke(new BasicStroke(2));
			g.setColor(Color.RED);
			for(int i = 1; i <= noOfSegments; i++) {
				int x1 = margin2 + (int)((i - 1) * hStep);
				int x2 = margin2 + (int)(i * hStep);
				int y1 = margin2 + sineValues[i-1];
				int y2 = margin2 + sineValues[i];
				g.drawLine(x1, y1, x2, y2);
			}
			g.setColor(Color.black);
			final int arrowSize = 5;
			Arrow arrow = new Arrow(arrowSize);
			arrow.drawArrowHead(g2, 0, getHeight() / 2., getWidth() - arrowSize, getHeight() / 2.);
			arrow.drawArrowHead(g2, margin2, getHeight(), margin2, arrowSize);
		}

		public void setCycles(int newCycles) {
			noOfSegments = SCALE_FACTOR * newCycles * 2;
			repaint();
		}
	}

	private JSlider createSlider(int min, int max, int value, SinePanel sinePanel) {
		JSlider jSlider = new JSlider(min, max, value);
		jSlider.setMajorTickSpacing(5);
		jSlider.setMinorTickSpacing(1);
		jSlider.setPaintTicks(true);
		jSlider.setPaintLabels(true);
		jSlider.setBackground(backgroundColor);
		jSlider.addChangeListener(e -> {
			int newCycles = ((JSlider)e.getSource()).getValue();
			sinePanel.setCycles(newCycles > 0 ? newCycles : 1);
		});
		return jSlider;
	}

	public SineWave() {

		final int initCycles = 5;
		setLayout(new BorderLayout(10, 10));
		setBorder(BorderFactory.createEmptyBorder(10, 10, 15, 10));
		setBackground(backgroundColor);

		final SinePanel sinePanel = new SinePanel(initCycles);

		add(sinePanel, BorderLayout.CENTER);
		add(createSlider(0, 20, initCycles, sinePanel), BorderLayout.PAGE_END);
	}

	public static void main(String[] args) {
		final int WIDTH = 700;
		final int HEIGHT = 400;
		SwingConsole.run(new SineWave(), WIDTH, HEIGHT);
	}
}

